@@ -1,4 +1,5 @@
 import fnmatch
+import io
 import logging
 import mimetypes
 import tempfile
@@ -6,6 +7,8 @@ import urllib
 from os import listdir
 from os.path import join
 
+import requests
+
 from kodexa.model import Document, DocumentMetadata
 
 
@@ -58,7 +61,7 @@ class FileHandleConnector:
         self.completed = False
 
     def get_source(self, document):
-        return open(join(self.file, document.metadata['connector_options']['file']), 'rb')
+        return open(document.metadata['connector_options']['file'], 'rb')
 
     def __iter__(self):
         return self
@@ -88,15 +91,23 @@ class UrlConnector:
         self.completed = False
 
     def get_source(self, document):
-        if 'headers' in document.metadata['connector_options']:
-            opener = urllib.request.build_opener()
-            for header in document.metadata['connector_options']['headers']:
-                opener.addheaders = [(header, document.metadata['connector_options']['headers'][header])]
-            urllib.request.install_opener(opener)
-        with tempfile.NamedTemporaryFile(delete=False) as tmp_file:
-            urllib.request.urlretrieve(document.metadata['connector_options']['url'], tmp_file.name)
 
-            return open(tmp_file.name, 'rb')
+        # If we have an http URL then we should use requests, it is much
+        # cleaner
+        if document.metadata['connector_options']['url'].startswith('http'):
+            response = requests.get(document.metadata['connector_options']['url'],
+                                    headers=document.metadata['connector_options']['headers'])
+            return io.BytesIO(response.content)
+        else:
+            if 'headers' in document.metadata.connector_options:
+                opener = urllib.request.build_opener()
+                for header in document.metadata.connector_options.headers:
+                    opener.addheaders = [(header, document.metadata['connector_options']['headers'][header])]
+                urllib.request.install_opener(opener)
+            with tempfile.NamedTemporaryFile(delete=True) as tmp_file:
+                urllib.request.urlretrieve(document.metadata['connector_options']['url'], tmp_file.name)
+
+                return open(tmp_file.name, 'rb')
 
     def __iter__(self):
         return self
