@@ -154,7 +154,7 @@ def tag_range(self, start_content_re, end_content_re, tag_name, type_re='.*', us
 
 def tag(self, tag_name, type_re=None, content_re=None,
         use_all_content=False, node_only=False, include_children=False,
-        fixed_position=None):
+        fixed_position=None, data=None):
     """
     This will tag (see Feature Tagging) the expression groups identified by the regular expression.
 
@@ -167,13 +167,15 @@ def tag(self, tag_name, type_re=None, content_re=None,
     :param node_only: Ignore the matching groups and tag the whole node
     :param include_children: Include recurse into children and tag where matching
     :param fixed_position: use a fixed position, supplied as a tuple i.e. - (4,10) tag from position 4 to 10 (default None)
+    :param data: Attach the a dictionary of data for the given tag
     """
 
     type_match = True
 
     if fixed_position:
         self.add_feature('tag', tag_name,
-                         Tag(fixed_position[0], fixed_position[1], self.content[fixed_position[0]:fixed_position[1]]))
+                         Tag(fixed_position[0], fixed_position[1], self.content[fixed_position[0]:fixed_position[1]],
+                             data))
     else:
         if type_re:
             type_re_compiled = re.compile(type_re)
@@ -181,7 +183,7 @@ def tag(self, tag_name, type_re=None, content_re=None,
 
         if type_match:
             if not content_re:
-                self.add_feature('tag', tag_name, [])
+                self.add_feature('tag', tag_name, Tag(data=data))
             else:
                 pattern = re.compile(content_re)
                 if not use_all_content:
@@ -195,11 +197,12 @@ def tag(self, tag_name, type_re=None, content_re=None,
                 match = pattern.match(content)
                 if match:
                     if node_only:
-                        self.add_feature('tag', tag_name, Tag())
+                        self.add_feature('tag', tag_name, Tag(data=data))
                     else:
                         for index, m in enumerate(match.groups()):
                             idx = index + 1
-                            self.add_feature('tag', tag_name, Tag(match.start(idx), match.end(idx), match.group(idx)))
+                            self.add_feature('tag', tag_name,
+                                             Tag(match.start(idx), match.end(idx), match.group(idx), data=data))
 
     if include_children:
         for child in self.children:
@@ -605,10 +608,6 @@ class CoreMixin:
     def get_renderer(document):
         return None
 
-    @staticmethod
-    def to_text(self):
-        return "[tags:" + ",".join(self.get_tags()) + "]"
-
     @staticmethod
     def apply_to(node):
         add_method_to_node(get_all_content, node)
@@ -641,7 +640,8 @@ class CoreMixin:
 
 class Tag(Dict):
 
-    def __init__(self, start=None, end=None, value=None):
+    def __init__(self, start=None, end=None, value=None, data=None):
         self.start = start
         self.end = end
         self.value = value
+        self.data = None
