@@ -1,30 +1,19 @@
 import json
-import os
 import uuid
 
 import msgpack
+from addict import Dict
 
 from kodexa.mixins import registry
-from kodexa.mixins.registry import get_mixin
 
 
-class DocumentMetadata(dict):
+class DocumentMetadata(Dict):
     """
     A flexible dict based approach to capturing metadata for the document
-
-    Sub-classes dict, and further allows attribute-like access to dictionary items.
-
-        >>> d = DocumentMetadata({'a': 1})
-        >>> d.a, d['a'], d.get('a')
-        (1, 1, 1)
-        >>> d.b = 2
-        >>> d.b, d['b']
-        (2, 2)
     """
 
     def __init__(self, *args, **kwargs):
-        super(DocumentMetadata, self).__init__(*args, **kwargs)
-        self.__dict__ = self
+        super().__init__(*args, **kwargs)
 
 
 class ContentNode(object):
@@ -65,25 +54,12 @@ class ContentNode(object):
         return f"ContentNode [type:{self.type}] ({len(self.get_features())} features, {len(self.children)} children) [" + str(
             self.content) + "]"
 
-    def to_text(self):
-        """
-        Convert this node  structure into a text representation,
-        which can be useful when trying to review the structure.
-
-
-            >>> node.to_text()
-        """
-        return DefaultDocumentRender(self.document).node_to_text(self, 0)
-
     def _repr_html_(self):
         return self.to_html()
 
     def to_html(self):
         return DocumentRender(self.document).render_node(self)
 
-    def to_mimetype(self):
-        return DocumentRender(self.document).render_node_mimetype(self)
-
     def to_json(self):
         """
         Convert this node structure into a JSON object
@@ -318,99 +294,13 @@ class DocumentRender:
 
     def __init__(self, document):
         self.document = document
-        self.default_renderer = DefaultDocumentRender(document)
-
-    def to_text(self):
-        return self.default_renderer.node_to_text(self.document.content_node, 0)
+        self.kodexa_render = KodexaRender()
 
     def to_html(self):
-        print(self.get_mixin_renderer())
-        return self.get_mixin_renderer().to_html()
-
-    def get_mixin_renderer(self):
-        renderer_dict = registry.get_renderers(self.document)
-        if renderer_dict:
-            return next(iter(renderer_dict.values()))
-        else:
-            return self.default_renderer
+        return self.kodexa_render.build_html(self.document, self.document.content_node)
 
     def render_node(self, node):
-        return self.get_mixin_renderer().render_node(node)
-
-    def to_mimetype(self):
-        return self.get_mixin_renderer().to_mimetype()
-
-    def render_node_mimetype(self, node):
-        return self.get_mixin_renderer().render_node_mimetype(node)
-
-
-class DefaultDocumentRender:
-
-    def __init__(self, document):
-        self.document = document
-
-    def to_text(self):
-        return self.node_to_text(self.document.content_node, 0)
-
-    def to_html(self):
-        return f"<h3>Document {self.document}</h3>"
-
-    def render_node(self, node):
-        out_list = []
-        self.get_node_text_format(node, 0, out_list)
-        return f"<h3>Content Node</h3><p>{self.node_to_text(node, 0)}"
-
-    def node_to_text(self, node, level):
-        out_list = []
-        self.get_node_text_format(node, level, out_list)
-        return os.linesep.join(out_list)
-
-    def get_node_text_format(self, node, level, out_list):
-        out_list.append("{}[type:{}]{}".format(" " * (4 * level), node.type, self.mixins_to_text(node)))
-
-        if node.content_parts:
-            out_list.append("{}{}".format(" " * (4 * level), self.get_content_parts(node)))
-
-        out_list.append("{}{}".format(" " * (4 * level), node.content))
-
-        for child in node.get_children():
-            self.get_node_text_format(child, level + 1, out_list)
-
-    def get_content_parts(self, node):
-        result = ""
-        for part in node.content_parts:
-            if isinstance(part, str):
-                result = result + part
-            # else:
-            #     result = result + "#c" + str(part)
-
-        return result
-
-    def mixins_to_text(self, node):
-        display = ""
-        for mixin in self.document.get_mixins():
-            if get_mixin(mixin):
-                text = get_mixin(mixin).to_text(node)
-                if text:
-                    display = display + get_mixin(mixin).to_text(node)
-
-        return display
-
-    def to_mimetype(self):
-        out_list = []
-        self.get_node_text_format(self.document.content_node, 0, out_list)
-        return self.prepare_mimetype_data('document', tuple(out_list))
-
-    def render_node_mimetype(self, node):
-        out_list = []
-        self.get_node_text_format(node, 0, out_list)
-        return self.prepare_mimetype_data('content_node', tuple(out_list))
-
-    def prepare_mimetype_data(self, data_type, data_list):
-        render_data = {'data_type': data_type, 'data': tuple(data_list)}
-        bundle = {}
-        bundle['application/vnd.kodexa.document+json'] = render_data
-        return bundle
+        return self.kodexa_render.build_html(self.document, node)
 
 
 class SourceMetadata(object):
@@ -423,9 +313,9 @@ class Document(object):
     """
 
     def __str__(self):
-        return f"MDocument {self.uuid} {self.metadata}"
+        return f"kdxa//{self.uuid}/{self.metadata}"
 
-    def __init__(self, metadata=None, content_node: ContentNode = None, connector=SourceMetadata()):
+    def __init__(self, metadata=None, content_node: ContentNode = None, source=SourceMetadata()):
         if metadata is None:
             metadata = DocumentMetadata()
         self.metadata: DocumentMetadata = metadata
@@ -436,10 +326,18 @@ class Document(object):
         self.exceptions = []
         self.log = []
         self.version = "1.0.0"
+        self.add_mixin('core')
+        self.source: SourceMetadata = source
 
         # Make sure we apply all the mixins
         registry.apply_to_document(self)
 
+    @classmethod
+    def from_text(cls, text):
+        new_document = Document()
+        new_document.content_node = new_document.create_node(type='text', content=text)
+        return new_document
+
     def get_root(self):
         """
         Get the root content node for the document (same as content_node)
@@ -449,16 +347,6 @@ class Document(object):
         """
         return self.content_node
 
-    def to_text(self):
-        """
-        Convert this document object structure into a text representation,
-        which can be useful when trying to review the structure.
-
-
-            >>> document.to_text()
-        """
-        return DocumentRender(self).to_text()
-
     def to_kdxa(self, file_path):
         """
         Write the document to the kdxa format (msgpack) which can be
@@ -488,7 +376,6 @@ class Document(object):
         """
         Convert this document object structure into a message pack
 
-
             >>> document.to_msgpack()
         """
         return msgpack.packb(self.to_dict(), use_bin_type=True)
@@ -522,9 +409,6 @@ class Document(object):
                 'log': self.log,
                 'uuid': self.uuid}
 
-    def to_mimetype(self):
-        return DocumentRender(self).to_mimetype()
-
     @staticmethod
     def from_dict(doc_dict):
         """
@@ -584,7 +468,8 @@ class Document(object):
         """
         registry.add_mixin_to_document(mixin, self)
 
-    def create_node(self, type=type, content=None, virtual=False, parent=None, index=0):
+    def create_node(self, type: str = type, content: str = None, virtual: bool = False, parent: ContentNode = None,
+                    index: int = 0):
         """
         Creates a new node for the document, this doesn't add the node to the document however
         it does ensure that any mixins that have been applied to the document will also be
@@ -605,3 +490,56 @@ class Document(object):
                 if callable(add_features_to_virtual_node):
                     add_features_to_virtual_node(content_node)
         return content_node
+
+    @classmethod
+    def from_file(cls, file):
+        file_document = Document()
+        file_document.metadata.connector = 'file-handle'
+        file_document.metadata.connector_options.file = file
+        return file_document
+
+    @classmethod
+    def from_url(cls, url, headers=None):
+        if headers is None:
+            headers = {}
+        url_document = Document()
+        url_document.metadata.connector = 'url'
+        url_document.metadata.connector_options.url = url
+        url_document.metadata.connector_options.headers = headers
+        return url_document
+
+
+class KodexaRender:
+    """
+    An implementation of a render that uses the KodexaJS
+    library to render the document
+
+    See https://github.com/kodexa-ai/kodexa.js
+    """
+
+    def build_node_html(self, node: ContentNode):
+        self.build_html(node.document, node)
+
+    def build_html(self, document: Document, node: ContentNode):
+        render_uuid = str(uuid.uuid4())
+        return """
+  <div id='kodexa-div-""" + render_uuid + """'></div> 
+  <script>
+
+require.config({
+    paths: {
+        'kodexa-lib-""" + render_uuid + """': 'https://unpkg.com/kodexajs@0.0.15/kodexa',
+        'jquery': '//ajax.googleapis.com/ajax/libs/jquery/3.5.1/jquery.min'
+    }
+});
+
+require(['kodexa-lib-""" + render_uuid + """','jquery'], function() {
+    kodexa.fromMap(""" + document.to_json() + """).then(kdxaDocument => {
+       let widget = kodexa.newDocumentWidget(kdxaDocument);
+       widget.attach($('#kodexa-div-""" + render_uuid + """'))
+       widget.render();
+    });
+    
+});
+</script>
+"""
