@@ -5,4 +5,4 @@ Sinks
 Sinks are the end-point of a Pipeline and allow for the final output of the pipeline to be either stored or written
 out
 """
-from .inmemory import *
+from .inmemory import InMemoryDocumentSink
